<?php
namespace Teknisa\EST\Controllers;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use Zeedhi\Framework\DataSource\DataSet;

use Teknisa\EST\Services\Est53000ParametrosDeEstoqueService;

class Est53000ParametrosDeEstoqueController {
    #default services
    protected $environment;
    protected $service;

    /* **************************************************
     * Construtor do Controller.
     *
     * @param Environment                        $environment
     * @param Est53000ParametrosDeEstoqueService $service
     */
    public function __construct($environment, Est53000ParametrosDeEstoqueService $service) {
        #initializes default services
        $this->environment   = $environment;
        $this->service       = $service;
    }

    public function deleteAllDays(DTO\Request\Filter $request, DTO\Response $response) {
        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            #tries to remove every persisted day
            $this->service->deleteAllDays($CDFILIAL, $NRORG);
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function getDataForGeral(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        try {
            #tries to get generic parametrization data
            $data = $this->service->getDataForGeral($NRORG);
            
            #returns data
            $response->addDataSet(new DataSet("paramestoque_geral", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function getDataForProdutoField(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForProdutoField($NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_produto_field", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function validateProductInsertion(DTO\Request\Filter $request, DTO\Response $response) {
        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #gets data
        $CDFILIAL     = $filters[0]['value'];
        $CDALMOXARIFE = $filters[1]['value'];
        $CDLOCALESTOQ = $filters[2]['value'];
        $CDPRODUTO    = $filters[3]['value'];
        $CDARVPROD    = $filters[4]['value'];
        $IDCURVAABC   = $filters[5]['value'];

        try {
            #validates product insertion
            $valid = $this->service->validateProductInsertion($CDPRODUTO, $CDARVPROD, $IDCURVAABC, $CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG);

            #returns validation
            $response->addDataSet(new DataSet("paramestoque_produto_validation", array($valid)));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Erro ao salvar os dados. ".$e->getMessage(), 500));
        }
    }

    public function saveProduct(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $CDFILIAL     = $row['CDFILIAL'];
        $CDALMOXARIFE = $row['CDALMOXARIFE'];
        $CDLOCALESTOQ = $row['CDLOCALESTOQ'];
        $IDCURVAABC   = $row['IDCURVAABC'];
        $CDPRODUTO    = $row['CDPRODUTO'];
        $NRORG        = $this->environment->getNrOrgTrab();
        try {

            if($row['__is_new']){
               $this->service->saveProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG);
            }else{
                $IDCURVAABC_OLD   = $row['IDCURVAABC_OLD'];
                $this->service->updateProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG, $IDCURVAABC_OLD);
            }

        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function deleteProdutosCurvaAbc(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $row['NRORG'] = $this->environment->getNrOrgTrab();
            $this->service->deleteProdutosCurvaAbc($row);
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }


    public function saveGeral(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->saveGeral($row);
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function saveUnidade(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->saveUnidade($row);
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }


    public function saveParametrosCurvaAbc(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->saveParametrosCurvaAbc($row);
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }


    public function getDataForProdutoWidget(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDALMOXARIFE = $filters[0]['value'];
        $CDLOCALESTOQ = $filters[1]['value'];
        $CDFILIAL     = $filters[2]['value'];
        $NRORG        = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_produto_widget", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function getDataForUnidade(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForUnidade($CDFILIAL, $NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_unidade", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}